
/*
 *
 * Copyright (C) 1999  Thomas Mirlacher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The author may be reached as dent@cosy.sbg.ac.at, or
 * Thomas Mirlacher, Jakob-Haringerstr. 2, A-5020 Salzburg,
 * Austria
 *
 *------------------------------------------------------------
 *
 */


#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#define MULTI_BUFFER 
#include <plugin/buf.h>

#define PRINT_DEBUG 0
#define dprintf(x) if (PRINT_DEBUG) fprintf (stderr, x)

/**
 *
 */

int DecoderBufInit (buf_t *buf)
{
	int i;

	for (i=0; i<_BUF_MAX; i++) {
		int s;

		pthread_mutex_init(&buf->ctrl[i].mutex, NULL);
		pthread_mutex_init(&buf->ctrl[i].mutex_slots_left, NULL);
		pthread_mutex_init(&buf->ctrl[i].mutex_new_data, NULL);
		pthread_cond_init (&buf->ctrl[i].cond_slots_left, NULL);
		pthread_cond_init (&buf->ctrl[i].cond_new_data, NULL);

		buf->ctrl[i].wait4new = 0;
		buf->ctrl[i].forbid_write = 0;

		for (s=0; s<BUF_DEPTH; s++) {
			buf->entry[i][s].data = NULL;
			buf->entry[i][s].data_len = 0;
			buf->entry[i][s].mem = NULL;
			buf->entry[i][s].mem_len = 0;
		}
	}

	return 0;
}


/**
 *
 */

int DecoderBufExit (buf_t *buf)
{
	int i;

	for (i=0; i<_BUF_MAX; i++) {
		DecoderBufFlush (buf, i);
		pthread_mutex_destroy (&buf->ctrl[i].mutex);
                pthread_mutex_destroy (&buf->ctrl[i].mutex_slots_left);
                pthread_mutex_destroy (&buf->ctrl[i].mutex_new_data);
                pthread_cond_destroy (&buf->ctrl[i].cond_slots_left);
                pthread_cond_destroy (&buf->ctrl[i].cond_new_data);
	}	

	return 0;
}


/**
 *
 */

void DecoderBufFlush (buf_t *buf, u_int buf_id)
{
	buf_entry_t entry;

	while (DecoderBufGet (buf, buf_id, &entry))
		free (entry.data);
	
}


// simple implementation of circular queues
/**
 *
 */

int DecoderBufPut (buf_t *buf, buf_entry_t *buf_entry)
{
	u_int write_index; 
	buf_entry_t *entry;
	u_int buf_id = buf_entry->buf_id;

//DENT ... just for debugging purposes ...
	switch (buf_entry->buf_id) {
		case BUF_AUDIO:
		case BUF_VIDEO:
			break;

		default:
			free (buf_entry->mem);
			return 0;
	}
// DENT

#if 0
	pthread_mutex_lock (&(buf->ctrl[buf_id].mutex));
	if (buf->ctrl[buf_id].wait4new) { // buffer empty
		pthread_cond_broadcast (&(buf->ctrl[buf_id].cond_new_data));
		buf->ctrl[buf_id].wait4new = 0;
	}
	pthread_mutex_unlock (&(buf->ctrl[buf_id].mutex));
#endif
	pthread_cond_signal (&(buf->ctrl[buf_id].cond_new_data));


	entry = &(buf->entry[buf_id][buf->ctrl[buf_id].write]);
	memcpy (entry, buf_entry, sizeof (buf_entry_t));

	write_index = (buf->ctrl[buf_id].write + 1) % BUF_DEPTH;

	if (write_index == buf->ctrl[buf_id].read) {	// buffer full
dprintf ("|");
		buf->ctrl[buf_id].forbid_write = 1;
		pthread_cond_wait (&(buf->ctrl[buf_id].cond_slots_left), &(buf->ctrl[buf_id].mutex_slots_left));
	}
dprintf (".");

	buf->ctrl[buf_id].write = write_index;

	return 0;
}


/**
 *
 */

int DecoderBufGet (buf_t *buf, u_int buf_id, buf_entry_t *entry)
{
	if (buf->ctrl[buf_id].read == buf->ctrl[buf_id].write) {// no new entry
		pthread_mutex_lock (&(buf->ctrl[buf_id].mutex));
		buf->ctrl[buf_id].wait4new = 1;
		pthread_mutex_unlock (&(buf->ctrl[buf_id].mutex));
		return -1;
	}

	memcpy (entry, &(buf->entry[buf_id][buf->ctrl[buf_id].read]), sizeof(buf_entry_t));


	buf->ctrl[buf_id].read = (buf->ctrl[buf_id].read + 1) % BUF_DEPTH;

	if (buf->ctrl[buf_id].forbid_write) {
#if 0
		buf->ctrl[buf_id].forbid_write = 0;
		pthread_cond_broadcast (&(buf->ctrl[buf_id].cond_slots_left));
#else
		u_int buf_fill;
dprintf ("-");

		if (buf->ctrl[buf_id].read <= buf->ctrl[buf_id].write)
			buf_fill = buf->ctrl[buf_id].write - buf->ctrl[buf_id].read;
		else
			buf_fill = (BUF_DEPTH - buf->ctrl[buf_id].read) + buf->ctrl[buf_id].write;

// wait a little bit, until we allow reading again ...
// (high water = 100%, low water = 50%)

		if (buf_fill <= BUF_DEPTH/2) {
			buf->ctrl[buf_id].forbid_write = 0;
			pthread_cond_signal (&(buf->ctrl[buf_id].cond_slots_left));
		}
#endif
	}

else dprintf ("+");
	return 0;
}
