/* 
 *  css_descramble.c
 *
 *  Released under the version 2 of the GPL.
 *
 *  Copyright 1999 Derek Fawcus
 *
 *  This file contains functions to descramble CSS encrypted DVD content
 *	
 */

/*
 *  Still in progress:  Remove the use of the bit_reverse[] table by recoding
 *                      the generation of LFSR1.  Finish combining this with
 *                      the css authentication code.
 *
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include "css.h"
#include "css_keys.h"
#include "css_tbl.h"


/**
 * this function is only used internally when decrypting title key
 */

static inline void _CSSKeyGenerate (u_char *key, u_char *in, u_char invert)
{
	u_char reorder[] = {
		0x05,0x00,0x01,0x02,0x03,
		0x04,0x00,0x01,0x02,0x03,
	};
	int i;
	u_char tmp1, tmp2, k[5];
	u_int tmp3, tmp4, tmp5, tmp6;

	tmp3 = in[0] | 0x100;
	tmp4 = in[1];

	tmp5 =	( (in[4] << 17)
		| (in[3] <<  9)
		| (in[2] <<  1))
		+ 8 - (in[2]&7);
	tmp5 =	  (table3[(tmp5    )&0xff] << 24)
		| (table3[(tmp5>> 8)&0xff] << 16)
		| (table3[(tmp5>>16)&0xff] <<  8)
		| (table3[(tmp5>>24)&0xff]);

	tmp6 = 0;

	for (i=0; i<5; i++) {
		tmp2 = table1[tmp4] ^ table1[(tmp3<<5)&0xff];
		tmp4 = tmp3>>1;
		tmp3 = ((tmp3&1)<<8) ^ tmp2;
		tmp2 = table3[tmp2];

		tmp1 = (((((((tmp5>>8)^tmp5)>>1)^tmp5)>>3)^tmp5)>>7);
		tmp5 = (tmp5>>8)|(tmp1<<24);

		tmp6 += (tmp1 ^ invert) + tmp2;
		k[i] = tmp6 & 0xff;
		tmp6 >>= 8;
	}

	for (i=9; i>=0; i--) 
		key[i%5] = k[i%5]^table0[key[i%5]]^key[reorder[i]];
}


/**
 * this function decrypts a title key with the specified disk key
 *
 * key_title: the unobfuscated title key (XORed with BusKey)
 * key_disk: the unobfuscated disk key (XORed with BusKey)
 *       2048 bytes in length (though only 5 bytes are needed, see below)
 * pkey: array of pointers to player keys and disk key offsets
 *
 *
 * use the result returned in key_title with css_descramble
 */

int CSSDecryptTitleKey (char *key_title, char *key_disk)
{
	static u_char tmp1[6], tmp3[6];
	int i;

	for (i=0; playerkeys[i].offset!=-1; ++i) {
		memcpy (tmp1, key_disk + playerkeys[i].offset, 5);
		_CSSKeyGenerate (tmp1, playerkeys[i].key, 0);

		memcpy (tmp3, key_disk, 5);
		_CSSKeyGenerate (tmp3, tmp1, 0);

		if (memcmp (tmp3, tmp1, 5) == 0) {
			fprintf (stderr, "Using Key %d\n", i+1);
			goto found_key;
		}
	}

	fprintf (stderr, "No suitable Key found.\n");
	return -1;

found_key:
	memcpy (tmp3, key_title, 5);
	_CSSKeyGenerate (tmp3, tmp1, 0xff);
	memcpy (key_title, tmp3, 5);

	return 0;
}


/**
 * this function does the actual descrambling
 *
 * buf_sec: encrypted sector (2048 bytes)
 * key: decrypted title key obtained from CSSDecryptTitleKey
 */

void CSSDescramble (u_char *buf_sec, char *key)
{
        u_char *buf_end = buf_sec + 2048;
        u_long tmp1, tmp2, tmp3, tmp4, tmp5, tmp6;

	if (!(buf_sec[20] & 0x30))
		return;

	tmp2 = 0x00;
	tmp4 = (*((u_long *)(key + 0x02))) ^ (*((u_long *)(buf_sec + 0x56)));
	tmp3 = tmp4 & 7;
	tmp4 = tmp4 * 2 + 8 - tmp3;

	tmp5 = key[1] ^ buf_sec[0x55];
	tmp6 = key[0] ^ buf_sec[0x54] | 0x100;

	buf_sec += 0x80;

	while(buf_sec <= buf_end) {
		tmp3 = table1[tmp5&0xff] ^ table1[(tmp6<<5)&0xff];
		tmp5 = tmp6 >> 1;
		tmp6 = ((tmp6 & 0x01) << 8) ^ tmp3;
		tmp3 = table4[tmp3&0xff];
		tmp1 = (((((((tmp4 >> 3) ^ tmp4) >> 1) ^ tmp4) >> 8) ^ tmp4) >> 5) & 255;
		tmp4 = (tmp4 << 8) | tmp1;
		tmp1 = table3[tmp1&0xff];
		tmp2 += tmp1 + tmp3;
		*buf_sec++ = table0[*buf_sec] ^ (tmp2 & 0xff);
		tmp2 >>= 8;
	}

	buf_end[-2048+20] &= 0x8f;
}
