/*  Livid 
 * 
 *  configuration-management, -file i/o
 *
 * Copyright (C) 1999  Thomas Mirlacher, Guenter Bartsch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "config_store.h"
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>

GData *pConfigData ;


gchar *get_config_string_value (gchar *strKey_) 
{
  return (gchar*) g_datalist_get_data  (&pConfigData,
					strKey_);
}


int get_config_int_value (gchar *strKey_, int nDefaultValue_) 
{
  gpointer pData ;
  gchar    *pDataString;
  int      nValue;

  pData = g_datalist_get_data  (&pConfigData,
				strKey_);

  if (pData == NULL) 
    {
      fprintf (stderr,  "config key %s not found\n", strKey_);
      

      set_config_int_value (strKey_, nDefaultValue_);
      
      return nDefaultValue_;
    } 
  else 
    {
      pDataString = (gchar*) pData;
      
      sscanf (pDataString, "%d", &nValue);
      
      //      fprintf (stderr,  "GET config key %s is %s int => %d\n", strKey_, pDataString, nValue);

      return nValue;

  }

}

void set_config_string_value (gchar *strKey_, gchar *strValue_) 
{
  gpointer pData;

  // free old data

  pData = g_datalist_get_data  (&pConfigData,
				strKey_);
  if (pData != NULL) 
    {
      g_datalist_remove_data (&pConfigData, strKey_);
      free (pData);
    }


  //  fprintf (stderr,  "CONFIG key STRING %s => %s \n", strKey_, strValue_);

  g_datalist_set_data(&pConfigData, strKey_, g_strdup(strValue_)) ;

}

void set_config_int_value (gchar *strKey_, int nValue_) 
{
  char     str[256];

  sprintf (str, "%d", nValue_);

  set_config_string_value (strKey_, str);
}



void init_config () 
{
  g_datalist_init (&pConfigData);

  read_config ();
}

void read_config_file (FILE *pFile_) 
{
  char line[256];
  char *pEqual, *pValue;
  gint nLength;

  while (fgets (&line[0], 256, pFile_)) 
    {
      nLength = strlen(line);
      line [nLength-1] = 0; // delete \n

      pEqual = strchr (line, '=');

      if (pEqual != NULL) 
	{
	  pValue = (char *) ( ((int) pEqual)+1);
	  *pEqual = 0;

	  //fprintf (stderr, "Value : %s\n", pValue);

	  set_config_string_value (g_strdup(&line[0]), pValue);
	}
    }
}

void read_config () 
{
  FILE *pFConfig;
  GString *strFilename;

  g_datalist_clear (&pConfigData);

  /*
   * read global configuration
   */

  /*
   * read user specific configuration
   */

  strFilename = g_string_new (getenv("HOME"));

  g_string_append (strFilename, "/.livid/config");

  //  fprintf (stderr,  "private config file %s\n", strFilename->str);

  pFConfig = fopen (strFilename->str, "r");

  if (pFConfig != NULL) {
    read_config_file (pFConfig);
    fclose (pFConfig);
  }

  g_string_free (strFilename, FALSE);
}

void  write_config_line  (GQuark key_id,
			  gpointer data,
			  gpointer user_data) 
{
  FILE *pFConfig;
  gchar *strKey;

  pFConfig = (FILE *) user_data;

  strKey = g_quark_to_string (key_id);

  //  fprintf (stderr, "Writing (fd=%d) %s = %s\n", fd, strKey, data);

  //  fprintf (stderr,"%s=%s\n",strKey,(gchar*) data);
  fprintf (pFConfig,"%s=%s\n",strKey,(gchar *) data);
}

void save_config () 
{
  FILE *pFConfig;

  GString *strFilename;
  struct stat stat_buffer;

  strFilename = g_string_new (getenv ("HOME"));
  g_string_append(strFilename, "/.livid");

  if (stat (strFilename->str, &stat_buffer) != 0) 
    {
      mkdir(strFilename->str, S_IRUSR | S_IWUSR | S_IXUSR);
 
    }

  g_string_free (strFilename, FALSE);


  strFilename = g_string_new (getenv ("HOME"));
  g_string_append(strFilename, "/.livid/config");

  pFConfig = fopen (strFilename->str, "w");

  if (pFConfig != NULL) {

    //    fprintf (stderr, "CONFIG Writing %s \n", strFilename->str);

    g_datalist_foreach (&pConfigData,
			write_config_line,
			pFConfig);

    fclose (pFConfig);
  }

  g_string_free (strFilename, FALSE);
}







