/*  Livid
 *
 * Copyright (C) 1999  Thomas Mirlacher, Guenter Bartsch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "gtk/gtk.h"
#include <gdk/gdk.h>
#include <gdk/gdkx.h>
#include <X11/Xlib.h>
#include <stdio.h>

#include "skinstore.h"
#include "config_store.h"
#include "pbutton.h"
#include "panel_gui.h"
#include "playlist.h"
#include "main.h"

#include <nav/nav.h>

extern struct player_global_struct player_global;
extern GtkWidget *widget_panel;
extern GtkWidget *widget_playlist;
extern GtkWidget *widget_config;

GtkWidget *mainwin;
GdkPixmap *mainwin_bg = NULL;
GdkGC     *mainwin_gc;
gint       nMainPX, nMainPY;
gboolean   bMainwin_moving = FALSE ;
GList     *mainwin_wlist = NULL;
PButton   *mainwin_close, *mainwin_eject;
PButton   *mainwin_pl, *mainwin_conf;
PButton   *mainwin_prev, *mainwin_rewind ;
PButton   *mainwin_stop, *mainwin_play, *mainwin_seek;
PButton   *mainwin_next;

GdkColor   mainwin_black, mainwin_blue;

GdkFont   *mainwin_lcdfont;

static gboolean inside_sensitive_widgets(gint x, gint y)
{
  return (inside_widget(x, y, mainwin_close) 
	  || inside_widget(x, y, mainwin_eject) 
	  || inside_widget(x, y, mainwin_pl) 
	  || inside_widget(x, y, mainwin_conf) 
	  || inside_widget(x, y, mainwin_prev)
	  || inside_widget(x, y, mainwin_rewind)
	  || inside_widget(x, y, mainwin_stop)
	  || inside_widget(x, y, mainwin_play)
	  || inside_widget(x, y, mainwin_seek)
	  || inside_widget(x, y, mainwin_next));
}

void mainwin_press(GtkWidget * widget, GdkEventButton * event, 
		   gpointer callback_data)
{
  fprintf (stderr, "mainwin_press\n");

  if ( (event->button == 1)
       && (event->type == GDK_BUTTON_PRESS)
       && !inside_sensitive_widgets(event->x, event->y))
    { // move panel

      gdk_window_raise(mainwin->window);
      gdk_pointer_grab(mainwin->window, FALSE, GDK_BUTTON_MOTION_MASK 
		       | GDK_BUTTON_RELEASE_MASK, GDK_NONE, 
		       GDK_NONE, GDK_CURRENT_TIME);

      bMainwin_moving = TRUE;

      nMainPX = event->x;
      nMainPY = event->y;


    } else {
      fprintf (stderr, "handle_press\n");
      handle_press_cb(mainwin_wlist, widget, event);
      draw_panel_window(FALSE);
    }
}

void mainwin_release(GtkWidget * widget, GdkEventButton * event, 
		     gpointer callback_data)
{
  gdk_pointer_ungrab(GDK_CURRENT_TIME);
  gdk_flush();

  bMainwin_moving = FALSE;

  handle_release_cb (mainwin_wlist, widget, event);

  draw_panel_window(FALSE);

}

void mainwin_motion(GtkWidget * widget, GdkEventMotion * event, 
		    gpointer callback_data)
{
  XEvent ev;
  gint i = 0;
  

  XSync(GDK_DISPLAY(), False);
  
  while (XCheckTypedEvent(GDK_DISPLAY(), MotionNotify, &ev))
    {
      event->x = ev.xmotion.x;
      event->y = ev.xmotion.y;
      i++;
    }

  if (bMainwin_moving)
    {
      gint mx, my, newx, newy;

      GdkModifierType modmask;
      gdk_window_get_pointer(NULL, &mx, &my, &modmask);

      newx = mx - nMainPX;
      newy = my - nMainPY;
      gdk_window_move(mainwin->window, newx, newy);

      set_config_int_value ("xpos", newx) ;
      set_config_int_value ("ypos", newy) ;

    }
  
  gdk_flush();

}

void mainwin_set_back_pixmap(void)
{
  gdk_window_set_back_pixmap(mainwin->window, mainwin_bg, 0);
  gdk_window_clear(mainwin->window);  
}

gint mainwin_client_event(GtkWidget *w,GdkEventClient *event, gpointer data)
{
  static GdkAtom atom_rcfiles = GDK_NONE;

  if (!atom_rcfiles)
    atom_rcfiles = gdk_atom_intern("_GTK_READ_RCFILES", FALSE);
  if(event->message_type == atom_rcfiles)
    {
      mainwin_set_back_pixmap();
      return TRUE;
      
    }
  return FALSE;
}

void panel_main(int nID_)
{
  //  printf ("Panel_main : %d \n", nID_);

  switch (nID_) 
    {
    case ID_CLOSE:
      // should only close panel, but for now...
      save_config();

      gtk_main_quit();
      //      exit(0);
      break;
    case ID_EJECT:
      // TODO TODO TODO
      break;
    case ID_PL:
      // show/hide playlist
      if (player_global.bPLShowing)
	gtk_widget_hide (widget_playlist);
      else
	gtk_widget_show (widget_playlist);
	
      player_global.bPLShowing = !player_global.bPLShowing;

      break;
    case ID_CONF:
      // show/hide config dialog
      if (player_global.bConfShowing)
	gtk_widget_hide (widget_config);
      else
	gtk_widget_show (widget_config);
	
      player_global.bConfShowing = !player_global.bConfShowing;

      break;
    }

}

void panel_update_ui () 
{
  if (navGetStatus () == STATUS_PLAY)
    pbutton_set_button_data ( mainwin_play,245, 0,245, 20);
  else
    pbutton_set_button_data ( mainwin_play,210, 0,210,20);

  draw_panel_window (TRUE);
}


void panel_play(int nID_)
{
  //	int title;
  //	int part;
  //	unsigned long int timecode;

  //	fprintf (stderr,"panel_play %d\n",nID_);

  switch (nID_) {
  case ID_PREV:
    playlist_play_prev();
    break;
  case ID_PLAY: {
    switch (navGetStatus ()) {
    case STATUS_FINISH:
    case STATUS_STOP:
      playlist_play_current();
      break;
    case STATUS_PAUSE_OFF:
    case STATUS_PLAY:
      navSetStatus (STATUS_PAUSE_ON);
      panel_update_ui();
      break;
    case STATUS_PAUSE_ON:
      navSetStatus (STATUS_PAUSE_OFF);
      panel_update_ui();
      break;
    }
    break;
  }
  case ID_STOP:
    if (navGetStatus () != STATUS_STOP) {
      navSetStatus (STATUS_STOP);
      panel_update_ui();
    }
    fprintf (stderr, "Stop\n");
    break;
  case ID_NEXT:
    playlist_play_next();
    break;
  }
}



void draw_panel_window(gboolean force)
{
  gboolean redraw;
  GdkPixmap *pSkinMain;
  gchar str[256];
  //  int title;
  //  int part;
  //  unsigned long int timecode;

  lock_widget_list(mainwin_wlist);

  /*
   * Main Skin
   */

  pSkinMain = get_skin_pixmap (SKIN_MAIN);

  gdk_draw_pixmap(mainwin_bg, mainwin_gc, pSkinMain, 
		  0, 0, 0, 0, 600, 90);
  
  /*
   * LC-Display (it isn't called liquid crystal display - Display ;)
   * GB: OKOK ;>
   */

  gdk_gc_set_foreground(mainwin_gc, &mainwin_black) ;
  gdk_draw_rectangle(mainwin_bg, mainwin_gc, 1, 
		     240,5,310,55);
  gdk_gc_set_foreground(mainwin_gc, &mainwin_blue) ;

  // =>Playlist  GetCurrentLocation (&title, &part, &timecode);

  sprintf (str, "Chapter %d %s", player_global.nCurChapter, player_global.strCurChapter);

  gdk_draw_string(mainwin_bg, mainwin_lcdfont, mainwin_gc, 
		  250, 40, str); 

  sprintf (str, "%s Title %d", player_global.strCurSourceName, player_global.nCurTitle);

  gdk_draw_string(mainwin_bg, mainwin_lcdfont, mainwin_gc, 
		  250, 20, str); 


  /*
   * draw widgets
   */


  draw_widget_list(mainwin_wlist, &redraw, TRUE);

  gdk_window_clear(mainwin->window);

  gdk_flush();

  unlock_widget_list(mainwin_wlist);

}

GtkWidget *setup_panel () {
  GdkColormap *colormap ;

  gdk_threads_enter();

  mainwin = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  gtk_widget_set_app_paintable(mainwin, TRUE);
  gtk_window_set_title(GTK_WINDOW(mainwin), "DVD Player");
  gtk_window_set_policy(GTK_WINDOW(mainwin), FALSE, FALSE, TRUE);

  gtk_widget_set_usize(GTK_WIDGET(mainwin), 600, 90);

  gtk_signal_connect (GTK_OBJECT(mainwin), "destroy",
		      GTK_SIGNAL_FUNC (gtk_main_quit), NULL);

  gtk_widget_set_events(mainwin, GDK_FOCUS_CHANGE_MASK 
			| GDK_BUTTON_MOTION_MASK | GDK_BUTTON_PRESS_MASK 
			| GDK_BUTTON_RELEASE_MASK | GDK_STRUCTURE_MASK);


  gtk_widget_realize(mainwin);

  init_skinstore (mainwin->window) ;

  gdk_window_set_decorations(mainwin->window, 0);

  mainwin_gc = gdk_gc_new(mainwin->window);

  /*
   * Font(s)
   */

  mainwin_lcdfont = gdk_font_load("-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*"); 

  /*
   * Colors
   */

  colormap = gdk_colormap_get_system() ;

  mainwin_black.red   = 0;
  mainwin_black.green = 0;
  mainwin_black.blue  = 0;

  gdk_colormap_alloc_color(colormap, &mainwin_black, FALSE, TRUE);

  mainwin_blue.red   = 6000;
  mainwin_blue.green = 6000;
  mainwin_blue.blue  = 65535;
  gdk_colormap_alloc_color(colormap, &mainwin_blue, FALSE, TRUE);

  /*
   * set back pixmap
   */

  mainwin_bg = gdk_pixmap_new(mainwin->window, 600, 90, 
			      gdk_visual_get_best_depth());

  gdk_window_set_back_pixmap(mainwin->window, mainwin_bg, 0);


  /*
   * buttons
   */
  
  mainwin_close = create_pbutton(&mainwin_wlist, mainwin_bg, mainwin_gc, 
		  		 560, 5, 35, 20,  0, 0,  0, 20, 
				 panel_main, SKIN_BUTTONS, ID_CLOSE);
  mainwin_eject = create_pbutton(&mainwin_wlist, mainwin_bg, mainwin_gc, 
		  		 200, 5, 35, 20, 35, 0, 35, 20, 
				 panel_main, SKIN_BUTTONS, ID_EJECT);

  mainwin_pl    = create_pbutton(&mainwin_wlist, mainwin_bg, mainwin_gc, 
		  		 500, 65, 35, 20, 315, 0, 315, 20, 
				 panel_main, SKIN_BUTTONS, ID_PL);
  mainwin_conf  = create_pbutton(&mainwin_wlist, mainwin_bg, mainwin_gc, 
		  		 535, 65, 35, 20, 350, 0, 350, 20, 
				 panel_main, SKIN_BUTTONS, ID_CONF);

  mainwin_prev   = create_pbutton(&mainwin_wlist, mainwin_bg, mainwin_gc, 
				  240, 65, 34, 20,105, 0,105,20, 
				  panel_play, SKIN_BUTTONS, ID_PREV);
  mainwin_rewind = create_pbutton(&mainwin_wlist, mainwin_bg, mainwin_gc, 
				  275, 65, 34, 20,140, 0,140,20, 
				  panel_play, SKIN_BUTTONS, ID_REWIND);
  mainwin_stop   = create_pbutton(&mainwin_wlist, mainwin_bg, mainwin_gc, 
				  310, 65, 34, 20,175, 0,175,20, 
				  panel_play, SKIN_BUTTONS, ID_STOP);
  mainwin_play   = create_pbutton(&mainwin_wlist, mainwin_bg, mainwin_gc, 
				  345, 65, 34, 20,210, 0,210,20, 
				  panel_play, SKIN_BUTTONS, ID_PLAY);
  mainwin_seek   = create_pbutton(&mainwin_wlist, mainwin_bg, mainwin_gc, 
				  380, 65, 34, 20,280, 0,280,20, 
				  panel_play, SKIN_BUTTONS, ID_SEEK);
  mainwin_next   = create_pbutton(&mainwin_wlist, mainwin_bg, mainwin_gc, 
				  415, 65, 34, 20, 70, 0, 70,20, 
				  panel_play, SKIN_BUTTONS, ID_NEXT);


  /*
   * Signals/Events
   */

  gtk_signal_connect(GTK_OBJECT(mainwin), "button_press_event", 
		     GTK_SIGNAL_FUNC(mainwin_press), NULL);
  gtk_signal_connect(GTK_OBJECT(mainwin), "button_release_event", 
		     GTK_SIGNAL_FUNC(mainwin_release), NULL);
  gtk_signal_connect(GTK_OBJECT(mainwin), "motion_notify_event", 
		     GTK_SIGNAL_FUNC(mainwin_motion), NULL);
  gtk_signal_connect(GTK_OBJECT(mainwin), "client_event",
		     GTK_SIGNAL_FUNC(mainwin_client_event), NULL);

  gdk_window_move(mainwin->window,  
		  get_config_int_value ("xpos", 200), 
		  get_config_int_value ("ypos", 0));

  gdk_threads_leave();

  return mainwin;

}
  








