
/*
 *
 * Copyright (C) 1999  Thomas Mirlacher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The author may be reached as dent@cosy.sbg.ac.at, or
 * Thomas Mirlacher, Jakob-Haringerstr. 2, A-5020 Salzburg,
 * Austria
 *
 *------------------------------------------------------------
 *
 */


/* This is needed for 64 bit file seek */
#define __USE_FILE_OFFSET64

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "plugin/input.h"

static int file_open (void);
static int file_close (void);
static int file_read (char *data, u_int len);
static int file_seek (u_long pos);

static struct {
	int fd;
} priv;

static plugin_input_t input = {
	PLUGIN_ID_INPUT,
	file_open,		// open
	file_close,		// close
	file_read,		// get_block
	file_seek,		// seek
	2048,			// blocksize
	&priv			// private data
};


/**
 *
 */

static int file_open (void)
{
	return 0;
}


/**
 *
 */

static int file_close (void)
{
	return 0;
}


/**
 *
 */

static int file_read (char *data, u_int len)
{
	return read (priv.fd, data, len);
}


/**
 *
 */

static int file_seek (u_long pos)
{
	lseek (priv.fd, pos, SEEK_SET); 

	return 0;
}


/**
 * Called upon init of input
 */

plugin_input_t *plugin_init (char *name)
{
	if ((priv.fd = open (name, O_RDONLY)) < 0) {
		perror ("open");
		return NULL;
	}

	return &input;
}


/**
 * Called when input is closed
 */

void plugin_exit (void)
{
	close (priv.fd);
}
